--------------------------------------------------------------------------------
-- SubModule SmallHistoryFile
-- Created   4/10/2005 7:28:13 PM
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;
--synopsys translate_off
library UNISIM;
use unisim.vcomponents.all;
--synopsys translate_on

entity SmallHistoryFile is port
   (
     d        : in    std_logic_vector(34 downto 0); -- data in
     b        : out   std_logic_vector(17 downto 0); -- data out
     CLK      : in    std_logic;                     -- clock
     A        : in    std_logic_vector(4 downto 0);  -- address
     HT       : in    std_logic;                     -- toggle order of history in memory
     W        : in    std_logic                      -- write enable
   );
end SmallHistoryFile;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture RTL of SmallHistoryFile is

   -- Component Declarations
   component RAM16X1D
      --synopsys translate_off
      generic
      (
        INIT : bit_vector := X"0000"
      );
      --synopsys translate_on
      port
      (
        A0    : in  STD_LOGIC;
        A1    : in  STD_LOGIC;
        A2    : in  STD_LOGIC;
        A3    : in  STD_LOGIC;
        D     : in  STD_LOGIC;
        DPO   : out STD_LOGIC;
        DPRA0 : in  STD_LOGIC;
        DPRA1 : in  STD_LOGIC;
        DPRA2 : in  STD_LOGIC;
        DPRA3 : in  STD_LOGIC;
        SPO   : out STD_LOGIC;
        WCLK  : in  STD_LOGIC;
        WE    : in  STD_LOGIC
      );
   end component RAM16X1D;

   attribute INIT : string;
   attribute INIT of all: label is "0000";
--   attribute INIT of bit_msb: label is "0000";

   -- Signal Declarations
   signal A4T, A4TN, MSB : std_logic;
   signal memin : std_logic_vector(35 downto 0);

begin
   MSB <= A(0);  -- selects the MSB/LSB (only used for read out)
   -- A(3..1) are for selecting the second order section, A(4) selects old/new
   A4T <= A(4) xor HT;  -- makes sure history values are interchanged everytime filter runs
   A4TN <= not A4T;      -- this one is for write, the above is for reading
    -- split 35 bit input into two 18 bit vectors; don't write garbage during simulation init
   memin(16 downto 0) <= To_StdLogicVector (To_bitvector (d(16 downto 0)));
   memin(17) <= '0';
   memin(35 downto 18) <= To_StdLogicVector (To_bitvector (d(34 downto 17)));

   -- memory is 18 times 2bit with output select for MSB/LSB and an output register
   memory: for i in 17 downto 0 generate
      signal mout : std_logic_vector(1 downto 0);
      signal mo : std_logic;
   begin
      -- select memory: dual port, 2 x 1 bit for LSB and MSB
      bit_lsb: component RAM16X1D
         port map
         (
            A0 => A(1),
            A1 => A(2),
            A2 => A(3),
            A3 => A4TN,
            D => memin(i),
            SPO => open,
            DPRA0 => A(1),
            DPRA1 => A(2),
            DPRA2 => A(3),
            DPRA3 => A4T,
            DPO => mout(0),
            WCLK => CLK,
            WE => W
         );
      bit_msb: component RAM16X1D
         port map
         (
            A0 => A(1),
            A1 => A(2),
            A2 => A(3),
            A3 => A4TN,
            D => memin(i+18),
            SPO => open,
            DPRA0 => A(1),
            DPRA1 => A(2),
            DPRA2 => A(3),
            DPRA3 => A4T,
            DPO => mout(1),
            WCLK => CLK,
            WE => W
         );
      -- select LSB/MSB of output
      lsbsel : process (MSB, mout) is
      begin
         case MSB is
            when '0' =>
               mo <= mout(0);
            when '1' =>
               mo <= mout(1);
            when others =>
               mo <= '0';
         end case;
      end process lsbsel;
      -- add register for output
      reg: process (CLK, mo) is
      begin
         if rising_edge (CLK) then
            b(i) <= mo;
         end if;
      end process reg;
   end generate memory;

end architecture RTL;
--------------------------------------------------------------------------------
